#!/usr/bin/env python
import re
import sys

class Packet(object):
    rx = re.compile('([^>]+)>([^,:]+(?:,[^,:]+)*):(.)(.*)$')
    def __init__(self, s):
        self.valid = False
        m = self.rx.match(s)
        if m is None:
            return
            #raise ValueError, "Unrecognized string %r" % s
        self.raw = s
        self.src, path, t, self.data = m.groups()
        path = path.split(',')
        self.dst = path[0]
        self.path = tuple(path[1:])
        self.call = ""
        self.sym = ""
        self.sym2 = ""
        self.lat = 0
        self.lon = 0
        self.course = 0
        self.speed = 0
        self.rangex = ""
        self.comptype = ""
        self.comment = ""
   

        if False:
            print("t = " + t)
        if t == '$':
            # Raw GPS data
            #print "raw $ is %s" % (self.data)
            pass
        elif t == "'":
            # Either old Mic-E data or current TM-D700 data
            #print "tm-d700 is %s" % (self.data)
            pass
        elif t == '/':
            # Position w/timestamp (no APRS messaging)
            #print "position data is %s" % (self.data)
            pass
        elif t == '@':
            # Position without timestamp (with APRS messaging)
            #print "messaging data is %s" % (self.data)
            pass
        elif t == '`':
            # Current Mic-E data
            #print("mic-e data is %s" % (self.data))
            self.decodeMicE(self.data)
            pass
        elif t == '!' or t == "=":
            # Posit without time, maybe compressed
            #print "base-91 data is %s" % (self.data)
            sym = self.data[0:1]
            if (sym == '/' or sym == '\\'):
                self.decode91(self.data)
            elif (sym >= '0' and sym <= '9'):
                self.decodeUncompressed(self.data)
                self.valid = True
            else:
                print >> sys.stderr, "not decoding unrecognized data %s " % (self.data)
        else:
            pass
            #print >> sys.stderr, "Unknown packet type %r" % t

    def getData(self):
        return "call=%s dst=%s sym=%s%s lat=%g lon=%g course=%s, speed=%s, range=%s, comptype=%s, comment=%s)" % (self.getCall(), self.getDst(), self.getSym(), self.getSym2(), self.getLat(), self.getLon(), self.getCourse(), self.getSpeed(), self.getRange(), self.getComptype(), self.getComment())

    def getCall(self):
        try:
            return self.src
        except:
            return ""

    def getDst(self):
        return self.dst

    def getSym(self):
        return self.sym
    
    def getSym2(self):
        return self.sym2
    
    def getLat(self):
        return self.lat
    
    def getLon(self):
        return self.lon
    
    def getCourse(self):
        return self.course
    
    def getSpeed(self):
        return self.speed
    
    def getRange(self):
        return self.rangex
    
    def getComptype(self):
        return self.comptype
    
    def getComment(self):
        return self.comment
    
    def getLatLon(self):
        return "%s,%s" % (self.getLat(), self.getLon())

    def decodeMicE(self, data):
        try:
            self.mice = data[0:6]
            self.sym = data[7]
            self.sym2 = data[6]
            self.mice_vendor = data[7:9]
            # if /> D7
            # if /] D700
            # if /`text...Mv message-capable
            # if /'text...Mv one-way
            # if /]123} then altitude and comment is after those 4 bytes
            # if /]146.52MHz...text then freq
            # if /]123}146.52MHz...text then freq and altitude
            # D710 ends with =
            # doesn't handle decoding position yet
            # see http://www.aprs.org/aprs12/mic-e-examples.txt
            # see parser.pm
            self.comment = data[13:]
        except Exception as err:
            try:
                print >> sys.stderr(err)
                print >> sys.stderr, ("Failed to decode MicE Data '" + data + "'" + " dest = " + self.dst + " ")
            except:
                print >> sys.stderr, "Failed to decode MicE Data '" + data + "'"

    def decodeUncompressed(self, data):
        #print "uncompressed " + data
        try:
            self.sym = data[8]
            self.sym2 = data[18]
            lon = int(data[0:2]) + float(data[2:7])/60.0
            self.lon = lon
            if data[8] == 'S': lon = -lon
            lat = int(data[9:12]) + float(data[11:17])/60.0
            if data[17] == 'W': lat = -lat
            self.lat = lat
            self.course = int(data[19:22])
            self.course = 0
            self.speed = int(data[23:26])
            self.speed = 0
            self.comment = data[27:]
        except:
            pass


    def decode91(self, data):
        self.sym = data[0:1]
        lon = 0
        lat = 0
        for c in data[1:5]:
            lat = lat*91 + (ord(c)-33)
        for c in data[5:9]:
            lon = lon*91 + (ord(c)-33)
        self.lat = 90.0 - lat / 380926.0
        self.lon = -180.0 + lon / 190463.0
        self.sym2 = data[9:10]
        c = data[10:11]
        s = data[11:12]
        self.comptype = data[12:13]
        self.comment = data[13:]

        self.course = 0
        self.speed = 0
        self.rangex = 0
        KM=1.85200
        MI=KM*0.621371192 
        if c == ' ':
            pass
        elif c >= '!' and c <= 'z':
            self.course = (ord(c) - 33)*4
            self.speed = int(10*(MI*1.08 ** (ord(s) - 33.0)))/10.0
        elif c == '{':
            self.rangex = 2 * 1.08 ** (ord(s) - 33.0)
        else:
            print >> sys.stderr, "invalid course %s" %(c)
        self.valid = True


    def __repr__(self):
        return '%s>%s:%s' % (self.src, ','.join((self.dst,)+self.path), self.data)
